var gulp          = require('gulp'),
    browserSync   = require('browser-sync'),
    sass          = require('gulp-sass'),
    prefix        = require('gulp-autoprefixer'),
    concat        = require('gulp-concat'),
    cp            = require('child_process'),
    cache         = require('gulp-cache'),
    imagemin      = require('gulp-imagemin'),
    gulpSettings  = require('./gulpsettings.json'),
    notify        = require('gulp-notify');
    plumber       = require('gulp-plumber'),
    gutil         = require('gulp-util'),
    runSequence   = require('run-sequence');

var onError = function (err) {
  gutil.beep();
  console.log(err);
};

/**
 * @task sass
 * Compile files from scss
 */
gulp.task('sass', function () {
  return gulp.src('assets/scss/*.scss')
  .pipe(plumber({ errorHandler: function(err) {
    notify.onError({
        title: "Gulp error in " + err.plugin,
        message:  err.toString()
    })(err);

    // play a sound once
    gutil.beep();
  }}))
  .pipe(sass())
  .pipe(prefix(['last 2 versions'], { cascade: true }))
  .pipe(gulp.dest('css'))
  .pipe(browserSync.reload({stream:true}))
});

// Optimizing Images
gulp.task('images', function() {
  return gulp.src('assets/images/**/*.+(png|jpg|jpeg|gif|svg)')
    // Caching images that ran through imagemin
    .pipe(cache(imagemin({
      interlaced: true,
    })))
    .pipe(gulp.dest('images'))
});


/**
 * @task watch
 * Watch scss files for changes & recompile
 * Clear cache when Drupal related files are changed
 */
gulp.task('watch', function () {
  gulp.watch(['assets/scss/*.scss', 'assets/scss/**/*.scss'], ['sass']);
});

/**
 * Default task, running just `gulp` will
 * compile Sass files, launch BrowserSync, watch files.
 */
gulp.task('default', ['watch']);

// Build
gulp.task('build', function(callback) {
  runSequence('sass',
              'images',
              callback);
});
