/**
 * @file
 * Charts functionality.
 */

(function ($, Drupal, once) {
  Drupal.behaviors.muserCharts = {
    attach: function (context, settings) {

      let charts = {};
      let chartSettings = {};

      let setChartHeight = function (element, chartInfo) {
        let height = chartInfo['height_override'];
        if (height) {
          $(element).attr('style', 'height: ' + height + 'px');
        }
      }

      let drawChart = function (id) {
        let data = google.visualization.arrayToDataTable(chartSettings[id]['data']);
        switch (chartSettings[id]['chart_type']) {
          case 'bar':
            charts[id] = new google.visualization.BarChart(document.getElementById(id));
            break;
          case 'combo':
            charts[id] = new google.visualization.ComboChart(document.getElementById(id));
            break;
        }
        charts[id].draw(data, chartSettings[id]['options']);
      }

      let updateChart = function (id, chartId, url) {
        $.get(url, {}, (chartInfo) => {
          if (!chartInfo['chart_id']) {
            return;
          }
          setChartHeight(this, chartInfo);
          if (chartInfo['chart']['data'].length === 0) {
            return;
          }
          let timeOut = chartInfo['timeout'];
          chartSettings[id] = chartInfo['chart'];
          drawChart(id);
          if (timeOut) {
            setTimeout(updateChart, timeOut * 1000, id, chartId, url);
          }
        });
      }

      $(once('charts--processed', '.muser-chart')).each( function () {

        let id = $(this).attr('id');
        let chartId = $(this).attr('data-chart-id');
        let queryParams = $(this).attr('data-chart-query-string');
        let passQueryParams = $(this).attr('data-chart-pass-query-parameters');
        let url = '/api/charts-data/' + chartId;
        if (queryParams) {
          url += '?' + queryParams;
        }
        else if (passQueryParams) {
          url += window.location.search;
        }
        let $noData = $('#no-data--' + id);
        $noData.show();

        $.get(url, {}, (chartInfo) => {
          if (!chartInfo['chart_id']) {
            return;
          }
          setChartHeight(this, chartInfo);
          if (chartInfo['chart']['data'].length === 0) {
            return;
          }
          $noData.hide();
          let timeOut = chartInfo['timeout'];
          chartSettings[id] = chartInfo['chart'];
          if (timeOut) {
            setTimeout(updateChart, timeOut * 1000, id, chartId, url);
          }
          google.charts.load('current', {packages: chartSettings[id]['packages']});
          google.charts.setOnLoadCallback(function () { drawChart(id); });
        });

      });

    }
  };
})(jQuery, Drupal, once);
